import textwrap

from mock import Mock
from nose.tools import assert_equal
from django.core.mail.message import EmailMessage

from ..mail import DjangoEmailBackend

class TestDjangoEmailBackend(object):
    def simple_test(self):
        dj_msg = EmailMessage(subject="Hello", body="World",
                              from_email="from@example.com",
                              to=["a@example.com", "b@example.com"],
                              cc=["cc@example.com"])
        mail_backend = DjangoEmailBackend()
        mail_backend.get_hub = Mock()
        mail_backend.send_messages([dj_msg])
        sent_messages = mail_backend.get_hub().send_many.call_args[0][0]
        mail_message = sent_messages[0].as_msg()["mail_message"]
        mail_message.pop("id")
        mail_message["body"] = "\n".join([
            line for line in mail_message["body"].splitlines()
            if not any(line.startswith(x) for x in ["Date", "Message-ID"])
        ])
        assert_equal(mail_message, {
            "from": "from@example.com",
            "to": ["a@example.com", "b@example.com"],
            "body":  textwrap.dedent("""\
                Content-Type: text/plain; charset="utf-8"
                MIME-Version: 1.0
                Content-Transfer-Encoding: quoted-printable
                Subject: Hello
                From: from@example.com
                To: a@example.com, b@example.com
                Cc: cc@example.com

                World"""
            )
        })




from django.db import connections, DEFAULT_DB_ALIAS, transaction

class table_lock(object):
    """ A context manager for PostgreSQL table locking::

        with table_lock(models.BankAccount, "EXCLUSIVE"):
            a = models.BankAccount.objects.get(id=1)
            b = models.BankAccount.objects.get(id=2)
            a.balance += 100
            b.balance -= 100
            a.save()
            b.save()
    """

    LOCK_MODES = set([
        "ACCESS SHARE",
        "ROW SHARE",
        "ROW EXCLUSIVE",
        "SHARE UPDATE EXCLUSIVE",
        "SHARE",
        "SHARE ROW EXCLUSIVE",
        "EXCLUSIVE",
        "ACCESS EXCLUSIVE",
    ])

    def __init__(self, model, mode, using=None):
        if mode.upper() not in self.LOCK_MODES:
            raise ValueError("invalid lock mode: %r" %(mode, ))
        self.model = model
        self.mode = mode
        self.using = using
        self.cxn = None
        self.txn = None

    def __enter__(self):
        assert self.cxn is None, "lock already acquired"
        self.cxn = transaction.get_connection(self.using)
        if not self.cxn.in_atomic_block:
            self.txn = transaction.atomic(using=self.using)
            self.txn.__enter__()
        self.cur = self.cxn.cursor()
        self.cur.execute("LOCK TABLE %s IN %s MODE" %(
            self.model._meta.db_table, self.mode,
        ))

    def __exit__(self, *exc):
        if self.txn is not None:
            self.txn.__exit__(*exc)



from django.shortcuts import render_to_response
from django.template import RequestContext

def r2r(request, template, dictionary=None, mimetype=None, content_type=None):
    """ A shortcut for 'render_to_response' which uses a RequestContext. """
    if mimetype is not None and content_type is None:
        content_type = mimetype
    context = RequestContext(request)
    return render_to_response(template, dictionary, context_instance=context,
                              content_type=content_type)



import sys
import logging
from itertools import groupby

from django.core import management
from django.conf import settings as s
from django.db import DEFAULT_DB_ALIAS, connections

from dwdj.management.base import BaseCommand, make_option

log = logging.getLogger(__name__.rpartition(".")[2])


class Command(BaseCommand):
    help = """
        Checks whether migrations needs to be run.
    """

    option_list = [
        make_option("-i", "--interactive", action="store_true", default=False),
        make_option("--use", default="auto", choices=["auto", "django", "south"],
            help="One of 'auto' (default), 'south', or 'django'."),
        make_option("--database", action="store", dest="database", default=DEFAULT_DB_ALIAS,
            help="Nominates a database to synchronize. Defaults to the 'default' database.",
        ),
    ]

    def handle(self, *args, **options):
        sys.exit(self.run(args, options))

    def run(self, args, options):
        use = options["use"]
        if use == "auto":
            use = "south" if "south" in s.INSTALLED_APPS else "django"
        return getattr(self, "run_%s" %(use, ))(args, options)

    def run_django(self, args, options):
        from django.db.migrations.loader import MigrationLoader
        connection = connections[options['database']]
        loader = MigrationLoader(connection)
        graph = loader.graph
        targets = graph.leaf_nodes()
        plan = []
        seen = set()

        # Generate the plan
        for target in targets:
            for migration in graph.forwards_plan(target):
                if migration not in seen:
                    plan.append(graph.nodes[migration])
                    seen.add(migration)

        unapplied = []
        for migration in plan:
            if (migration.app_label, migration.name) not in loader.applied_migrations:
                unapplied.append(migration)

        quiet = options.get("quiet")
        if not unapplied:
            if not quiet:
                print "All available migrations have been applied."
            return 0

        if not quiet:
            print "FOUND UNAPPLIED MIGRATIONS"
            print "=========================="
            for migration in sorted(unapplied, key=lambda x: (x.app_label, x.name)):
                print "%s: %s" %(migration.app_label, migration.name)

        if options.get("interactive"):
            print
            resp = raw_input("Apply all of these migrations? [y/N] ").strip().lower()
            if resp in ["y", "yes"]:
                management.call_command("migrate")
                return 0
            print "NOT applying migrations."

    def run_south(self, args, options):
        applied_dict = dict(self.get_applied_migrations())
        available = self.get_available_migrations()
        unapplied = []
        for app_name, available_migrations in available:
            unapplied_migrations = available_migrations - applied_dict.get(app_name, set())
            if unapplied_migrations:
                unapplied.append((app_name, unapplied_migrations))

        quiet = options.get("quiet")

        if not unapplied:
            if not quiet:
                print "All available migrations have been applied."
            return 0

        if not quiet:
            print "FOUND UNAPPLIED MIGRATIONS"
            print "=========================="
            for app_name, unapplied_migrations in sorted(unapplied):
                print "%s: %s" %(app_name, ", ".join(sorted(unapplied_migrations)))

        if options.get("interactive"):
            print
            resp = raw_input("Apply all of these migrations? [y/N] ").strip().lower()
            if resp in ["y", "yes"]:
                management.call_command("migrate")
                return 0
            print "NOT applying migrations."
        return 1

    def get_applied_migrations(self):
        from south.models import MigrationHistory
        return [
            (app_name, set(mh.migration for mh in mhistories))
            for (app_name, mhistories) in 
            groupby(
                MigrationHistory.objects
                    .filter(applied__isnull=False)
                    .order_by("app_name"),
                lambda m: m.app_name,
            )
        ]

    def get_available_migrations(self):
        from south.migration.base import all_migrations
        return [
            (app_migrations.app_label(), set(m.name() for m in app_migrations))
            for app_migrations in all_migrations()
        ]



import calendar
from datetime import datetime

def parse_date(datestr):
    """ Parses a date which has come from the web. Expects "%Y-%m-%d". """
    return datetime.strptime(datestr, "%Y-%m-%d")

def parse_datetime(datetimestr):
    """ Parses a date and time which has come from the web.
        Expects "%Y-%m-%d %H:%M:%S". """
    return datetime.strptime(datetimestr, "%Y-%m-%d %H:%M:%S")

def dtfromtimestamp(timestamp):
    """ Returns a datetime from timestamp ``timestamp``.

        >>> dtfromtimestamp(1234.5)
        datetime.datetime(1970, 1, 1, 0, 20, 34, 500000)
        >>> """
    return datetime.utcfromtimestamp(timestamp)

def timestampfromdt(dt):
    """ Returns a timestamp from datetime ``dt``.

        >>> timestampfromdt(datetime(1970, 1, 1, 0, 20, 34, 500000))
        1234.5
        >>> """
    return calendar.timegm(dt.timetuple()) + (dt.microsecond / 1000000.0)



import logging
from datetime import datetime

from django.core import urlresolvers
from django.conf import settings as s
from django.http import Http404, HttpResponsePermanentRedirect


class ActualHTTPMethodMiddleware(object):
    """ Allow an 'actual_method' GET paramter to modify the HTTP request
        parameter (for JavaScript and friends which don't really support
        PUT/DELETE). """

    def process_request(self, request):
        _old_mutable = request.GET._mutable
        request.GET._mutable = True
        try:
            actual_method = request.GET.pop("__actual_method", None)
            if actual_method is not None:
                request.method = actual_method[0].upper()
        finally:
            request.GET._mutable = _old_mutable


class SetRemoteAddrFromForwardedFor(object):
    """ Sets ``request.META["REMOTE_ADDR"]`` based on the value of the
        ``X-Forwarded-For`` header. """

    def process_request(self, request):
        try:
            http_x_forwarded_for = request.META["HTTP_X_FORWARDED_FOR"]
        except KeyError:
            return
        # If the x-forwarded-for header exists when Apache gets the request,
        # it will stick its address onto the end:
        #     X-Forwarded-For: user-supplied-value, actual-user-address
        # Only grab the last address, which will be the one added by Apache.
        remote_addr = http_x_forwarded_for.rsplit(",", 1)[-1].strip()
        request.META["REMOTE_ADDR"] = remote_addr


class AccessLogMiddleware(object):
    """ Log Apache-style access logs using Python's logging module. """

    access_log = logging.getLogger("accesslog.access")
    error_log = logging.getLogger("accesslog.error")

    def __init__(self):
        logging.getLogger("django.request").setLevel(logging.CRITICAL)

    def format_log(self, request, status_code, body_bytes):
        env = request.META
        user = getattr(request, "user", None)
        username = user and user.username or "-"
        su_state = getattr(request, "su_state", None)
        if su_state is not None and su_state.old_user is not None:
            username = '"%s acting-as %s"' %(
                su_state.old_user.username, username,
            )
        return ('%s %s %s [%s] "%s %s %s" %s %s' %(
            env.get("REMOTE_ADDR"),
            hex(id(request))[-6:],
            username,
            datetime.utcnow().strftime("%d/%b/%Y:%H:%M:%S -0000"),
            env.get("REQUEST_METHOD"),
            env.get("PATH_INFO"),
            env.get("SERVER_PROTOCOL"),
            status_code,
            body_bytes or "-",
        )).encode("utf-8")

    def process_response(self, request, response):
        body_bytes_func = getattr(response.content, "__len__", None)
        if body_bytes_func is not None:
            body_bytes = body_bytes_func()
        else:
            body_bytes = 0
        status = response.status_code
        formatted = self.format_log(request, status, body_bytes)
        if status < 500:
            self.access_log.info(formatted)
        else:
            self.access_log.warning(formatted)
        return response

    def process_exception(self, request, exception):
        if isinstance(exception, Http404):
            # Note: this will be caught by Django and handled by
            # process_response.
            return
        self.error_log.exception(self.format_log(request, 500, 0))


class RemoveTrailingSlashMiddleware(object):
    """ The opposite of Django's ``APPEND_SLASH``, removes a trailing slash
        from URLs when appropriate. """

    def process_request(self, request):
        old_path = request.path
        if not old_path.endswith("/") or old_path == "/":
            return

        urlconf = getattr(request, "urlconf", None)

        if urlresolvers.is_valid_path(request.path_info, urlconf):
            return

        if not urlresolvers.is_valid_path(request.path_info.rstrip("/"), urlconf):
            return

        new_path = request.path.rstrip("/")

        if s.DEBUG and request.method == 'POST':
            raise RuntimeError((
                "You called this URL via POST, but the URL should not end "
                "in a slash and RemoveTrailingSlashMiddleware is active. "
                "I can't redirect to the slash URL while maintaining POST data. "
                "Change your form to point to %s (no trailing slash)."
            ) %(new_path, ))

        query = request.environ.get("QUERY_STRING")
        if query:
            new_path += "?" + query
        return HttpResponsePermanentRedirect(new_path)



from mock import Mock
from nose.tools import assert_equal

from ensi_common.testing import parameterized

from ..urls import include_factory


dummy = object()

class TestIncludeFactory(object):
    tests = [
        ("foo", "foo"),
        (".foo", "p.k.g.foo"),
        ("..foo", "p.k.foo"),
        (dummy, dummy),
    ]

    @parameterized(tests)
    def test_include_factory(self, input, expected):
        m = Mock()
        include = include_factory("p.k.g", includefunc=m)
        include(input)

        assert_equal(m.call_args, ((expected, ), {}))



import sys
import logging
from functools import partial
from optparse import make_option, OptionParser

from django.dispatch import Signal
from django.utils.functional import cached_property
from django.core.management.base import BaseCommand as DjBaseCommand
from django.core.management.base import (
    CommandError, OutputWrapper, handle_default_options,
)

from dwdj.strutil import dedent

CommandError

signal_pre_run = Signal(providing_args=["argv"])
signal_post_run = Signal(providing_args=["status"])

class BaseCommand(DjBaseCommand):
    """ A less terrible base class for Djagno management commands.

        Changes:

        * Tracebacks are turned on by default
        * Verbosity is controlled by a counter instead of a number (ex,
          ``-vvv`` instead of ``-v3``) and are used to set up logging levels
          too (disable by setting ``logging_enabled = False``).
        * Will override a named logger (see ``logging_handler_name``) with the
          level and formatter settings from the command line (or add a new
          stderr logger if no matching logger is found).
        * Subclassess can set ``option_list`` directly - the base options are
          stored in ``base_option_list`` and merged with ``get_option_list``.
        * The ``self.log`` attribute will be set to a logger configured to use
          the name from ``self.get_log_name()`` (which defaults to the fully
          qualified name of the module containing the command).
        * If ``handle`` returns an integer it will be treated as a numeric exit
          status. If a ``str`` or ``unicode`` is returned it will be treated
          "normally" and 0 will be returned.
        * If ``handle`` raises an exception that exception will be logged
          (unless ``self.log_exc`` is ``False``) and the command will exit
          with a status of 1.

        Example::

            from dwdj.management.base import BaseCommand, make_option

            class MyCommand(BaseCommand):
                option_list = [
                    make_option("-f", "--foo", action="store_true"),
                ]

                def handle(self, *args, **options):
                    if options.get("foo"):
                        return 1
                    return 0
    """

    # For now, tell Django to use the legacy optparse option parsing. At some
    # point we'll need to update all the option parsing so it's done the fancy
    # new way with argparse, but that day is not today.
    use_argparse = False

    base_option_list = [
        make_option('-q', '--quiet', action="store_const", const=-1, dest="verbosity"),
        make_option('-v', '--verbose', action="count", default=0, dest="verbosity"),
        make_option('--verbose-log', action="store_true", help=dedent("""
            Use a more verbose logging format.
        """)),
        make_option('--settings', help=dedent("""
            The Python path to a settings module, e.g.
            "myproject.settings.main". If this isn\'t provided, the
            DJANGO_SETTINGS_MODULE environment variable will be used.')
        """)),
        make_option('--pythonpath', help=dedent("""
            A directory to add to the Python path, e.g.
            "/home/djangoprojects/myproject".
        """)),
        make_option('--traceback', action='store_true', default=True, help=dedent("""
            Log complete exception traceback, not just exception and message.
        """)),
        make_option('--no-traceback', action='store_false', dest="traceback", help=dedent("""
            Log only exception messages, not complete tracebacks.
        """))
    ]

    option_list = []

    logging_enabled = True
    logging_handler_name = "stderr"
    logging_format = '%(levelname)s [%(name)s]: %(message)s'
    logging_format_verbose =  '%(asctime)s %(levelname)s [%(processName)s:%(threadName)s:%(name)s]: %(message)s'
    log_exc = True

    def get_option_list(self):
        return self.base_option_list + self.option_list

    def create_parser(self, prog_name, subcommand):
        """
        Create and return the ``OptionParser`` which will be used to
        parse the arguments to this command.

        """
        parser = OptionParser(prog=prog_name,
                              usage=self.usage(subcommand),
                              version=self.get_version(),
                              option_list=self.get_option_list())
        original_parse = parser.parse_args
        parser.parse_args = partial(self._override_parse_args, parser,
                                    original_parse)
        return parser

    def _override_parse_args(self, parser, original_parse, *argv, **kwargs):
        options, args = original_parse(*argv, **kwargs)
        self.logging_setup(options)
        if options.verbosity < 0:
            options.traceback = False
        return (options, args)

    def logging_setup(self, options):
        verbosity = options.verbosity
        level = (
            logging.CRITICAL if verbosity < 0 else
            logging.ERROR if verbosity == 0 else
            logging.WARNING if verbosity == 1 else
            logging.INFO if verbosity == 2 else
            logging.DEBUG
        )
        format = (
            self.logging_format if not options.verbose_log else
            self.logging_format_verbose
        )
        logger, handler = self.logging_get_handler()
        formatter = logging.Formatter(format)
        handler.setFormatter(formatter)
        if logger.level > level:
            logger.setLevel(level)
        handler.setLevel(level)

    def get_log_name(self):
        return type(self).__module__

    @cached_property
    def log(self):
        return logging.getLogger(self.get_log_name())

    def logging_get_handler(self):
        logger = logging.getLogger("")
        for handler in logger.handlers:
            if handler.name == self.logging_handler_name:
                return logger, handler
        handler = logging.StreamHandler(sys.stderr)
        handler.name = self.logging_handler_name
        logger.addHandler(handler)
        return logger, handler

    def run_from_argv(self, argv):
        """
        Set up any environment changes requested (e.g., Python path
        and Django settings), then run this command. If the
        command raises a ``CommandError``, intercept it and print it sensibly
        to stderr.
        """
        signal_pre_run.send(self, argv=argv)
        parser = self.create_parser(argv[0], argv[1])
        options, args = parser.parse_args(argv[2:])
        handle_default_options(options)
        try:
            result = self.execute(*args, **options.__dict__)
        except SystemExit as e:
            signal_post_run.send(self, status=e.code)
            raise
        except BaseException:
            result = self.handle_execute_exc(options)
            if result is None:
                result = 1
        status = result or 0
        signal_post_run.send(self, status=status)
        sys.exit(status)

    def handle_execute_exc(self, options):
        if not self.log_exc:
            return 1
        if options.traceback:
            self.log.exception("Exception running command:")
        else:
            exc_info = sys.exc_info()
            self.log.error("%s: %s", exc_info[0].__name__, exc_info[1])
        return 1

    def execute(self, *args, **options):
        """
        Try to execute this command, performing model validation if
        needed (as controlled by the attribute
        ``self.requires_model_validation``, except if force-skipped).
        """

        # Switch to English, because django-admin.py creates database content
        # like permissions, and those shouldn't contain any translations.
        # But only do this if we can assume we have a working settings file,
        # because django.utils.translation requires settings.
        saved_lang = None
        self.stdout = OutputWrapper(options.get('stdout', sys.stdout))
        self.stderr = OutputWrapper(options.get('stderr', sys.stderr), self.style.ERROR)

        if self.can_import_settings:
            from django.utils import translation
            saved_lang = translation.get_language()
            translation.activate('en-us')

        try:
            if getattr(self, "requires_model_validation", False) and not options.get('skip_validation'):
                self.validate()
            result = self.handle(*args, **options)
            if isinstance(result, basestring):
                if self.output_transaction:
                    # This needs to be imported here, because it relies on
                    # settings.
                    from django.db import connections, DEFAULT_DB_ALIAS
                    connection = connections[options.get('database', DEFAULT_DB_ALIAS)]
                    if connection.ops.start_transaction_sql():
                        self.stdout.write(self.style.SQL_KEYWORD(connection.ops.start_transaction_sql()))
                self.stdout.write(result)
                if self.output_transaction:
                    self.stdout.write('\n' + self.style.SQL_KEYWORD("COMMIT;"))
                result = 0
            return result
        finally:
            if saved_lang is not None:
                translation.activate(saved_lang)



from django.test import TestCase
from django.db import transaction

from helper_project.models import HelperModel

from ..transaction import table_lock

class TableLockTestCase(TestCase):
    def test_table_lock_no_transaction(self):
        with table_lock(HelperModel, "EXCLUSIVE"):
            pass

    def test_table_lock_in_transaction(self):
        with transaction.atomic():
            with table_lock(HelperModel, "EXCLUSIVE"):
                pass



class SwappableCache(object):
    """ A Django cache backend which allows the *true* backend to be swapped at
        runtime. Especially useful for tests, and moderately optimized so it
        shouldn't be a problem to use in production.

        To use ``SwappableCache``, add this to ``settings.py``::

            CACHES = {
                "default": {
                    "BACKEND": "dwdj.cache.SwappableCache",
                    "LOCATION": "redis_cache",
                },
                "redis_cache": {
                    "BACKEND": "redis_cache.cache.RedisCache",
                    "LOCATION": "localhost:,
                    "KEY_PREFIX": "cache",
                },
            }

        And then from each test::

            from django.core import cache as django_cache
            from django.conf import settings as s

            def setUp(self):
                self.old_caches = s.CACHES
                self.old_backend = django_cache.cache.get_backend()
                new_caches = dict(s.CACHES)
                new_caches.update({
                    "default": {
                        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                    },
                    "locmem": {
                        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                    },
                })
                s.CACHES = new_caches
                django_cache.cache.set_backend("locmem")
                django_cache.cache.clear()

            def tearDown(self):
                s.CACHES = self.old_caches
                django_cache.cache.backend.set_backend(self.old_backend)

        Note that both ``django.core.cache.cache`` must be fiddled with because
        it is created once at load time **and** ``settings.CACHES`` must be
        fiddled with because ``django.core.cache.get_cache`` doesn't do any
        caching, so subsequent calls to ``get_cache(...)`` will use
        ``settings.CACHES``.
    """

    def __init__(self, host, *args, **kwargs):
        self._backend = None
        self._backend_attrs = []
        self.default_backend = host

    def set_backend(self, new_backend):
        from django.core.cache import get_cache as django_get_cache
        if isinstance(new_backend, basestring):
            new_backend = django_get_cache(new_backend)
        self._backend = new_backend
        for attr in self._backend_attrs:
            try:
                delattr(self, attr)
            except AttributeError:
                pass

    def get_backend(self):
        self.set_backend(self.default_backend)
        return self._backend

    def __getattr__(self, attr):
        backend = (self._backend or self.get_backend())
        try:
            value = getattr(backend, attr)
        except AttributeError:
            return object.__getattr__(self, attr)
        self._backend_attrs.append(attr)
        setattr(self, attr, value)
        return value

    def __contains__(self, key):
        backend = (self._backend or self.get_backend())
        return key in backend



import sys
import logging

from dwdj.management.base import BaseCommand, CommandError

log = logging.getLogger("script")

HELP = """\
Sets up the Django environment then runs a script as if it were invoked
directly. Useful for situations where a script needs access to models,
settings, etc, but creating a management command is undesirable.

For example::

    $ ./manage.py script cleanup.py
    Cleanup complete
    $ cat cleanup.py
    from django.conf import settings as s
    from myapp.models import MyModel
    MyModel.cleanup(options=s.CLEANUP_OPTIONS)
    print "Cleanup complete"
"""

class Command(BaseCommand):
    help = HELP

    def handle(self, *args, **options):
        if not args:
            raise CommandError("Usage: script SCRIPT_NAME [SCRIPT_ARGS ...]")
        script = args[0]
        old_argv = list(sys.argv)
        try:
            sys.argv[:] = [
                "%s %s" %(sys.argv[0], script),
            ] + list(args[1:])
            script_globals = {
                "__package__": None,
                "__doc__": None,
                "__name__": "__main__",
                "__builtins__": globals()["__builtins__"],
                "__file__": script,
            }
            execfile(script, script_globals, script_globals)
        finally:
            sys.argv[:] = old_argv



import time
import random

from django.forms import widgets
from django.db import models as m

from .strutil import to36


try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([], ["^fi\.dj\.fields\.IDField"])


class IDField(m.Field):
    """ An ID field which is a 12 byte string, which is a base36 encoded 62
        bit integer, where the leading bits are time, and the trailing bits
        are random. """
    description = "A unique ID 12 byte string, suitable for use as a unique ID"
    max_length = 12

    # Note: because of http://code.djangoproject.com/ticket/13696, IDField will
    # do Bad Things when it's used with an inline admin.

    @classmethod
    def new(cls):
        """ Returns a unique 62 bit ID which is based on a random number and
            the current time. """
        # Truncate the current unix time to 30 bits...
        curtime = int(time.time()) & ((1<<30)-1)
        # ... then slap some random bits on the end.
        # Do this to help the database maintain temporal locality.
        # (it's possible that these should be swapped - with the
        # random bits coming first and the time bits coming second)
        return to36((curtime << 32) | random.getrandbits(32))

    def __init__(self, primary_key=True, auto=True, **kwargs):
        self.auto = auto
        super(IDField, self).__init__(primary_key=primary_key,
                                      default=None,
                                      max_length=self.max_length,
                                      blank=True)

    def db_type(self, connection):
        return "CHAR(%s)" %(self.max_length, )

    def to_python(self, value):
        if isinstance(value, basestring):
            return value
        elif isinstance(value, (int, long)):
            return to36(value)
        elif value is None:
            # When an object is being deleted, its primary key is null'd
            return value
        raise TypeError("Bad ID Field value: %r (bad type)" %(value, ))

    def pre_save(self, model_instance, add):
        # Note: only set on save because Django's admin (or possibly just the
        # 'TabularInline' admin) gets upset when the primary key changes (as it
        # would if a 'default' value was set (eg, 'default=self.new').
        cur_val = super(IDField, self).pre_save(model_instance, add)
        if not cur_val and self.auto:
            cur_val = self.new()
            setattr(model_instance, self.attname, cur_val)
        if cur_val and len(cur_val) > self.max_length:
            raise ValueError("invalid %r on %r: %r is longer than %r"
                             %(self.attname, model_instance, cur_val,
                               self.max_length))
        return cur_val

    def get_prep_value(self, value):
        if isinstance(value, (int, long)):
            # This can happen when using 'dumpdata', for some reason
            return to36(value)
        return value

    def formfield(self, **kwargs):
        defaults = {'widget': ReadOnlyWidget}
        defaults.update(kwargs)
        return super(IDField, self).formfield(**defaults)


class ReadOnlyWidget(widgets.Widget):
    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, type='hidden', name=name)
        if value != '':
            final_attrs['value'] = widgets.force_text(value)
        return widgets.format_html('<input{0} />{1}',
                                   widgets.flatatt(final_attrs),
                                   value)



#!/usr/bin/env python

import os
import sys

from setuptools import setup, find_packages

os.chdir(os.path.dirname(sys.argv[0]) or ".")

try:
    long_description = open("README.rst", "U").read()
except IOError:
    long_description = "See https://github.com/wolever/dwdj"

import dwdj
version = "%s.%s.%s" %dwdj.__version__

setup(
    name="dwdj",
    version=version,
    url="https://github.com/wolever/dwdj",
    author="David Wolever",
    author_email="david@wolever.net",
    description="""
        A collection of useful django utilities.
    """,
    long_description=long_description,
    packages=find_packages(),
    license="BSD",
    classifiers=[ x.strip() for x in """
        Development Status :: 4 - Beta
        Intended Audience :: Developers
        License :: OSI Approved :: BSD License
        Natural Language :: English
        Operating System :: OS Independent
        Programming Language :: Python
        Topic :: Utilities
    """.split("\n") if x.strip() ],
)



import os

from django import template
from django.utils.html import escape
from django.utils.safestring import mark_safe

from django.conf import settings as s
from django.contrib.staticfiles.finders import find as find_staticfiles

register = template.Library()

def _stat(file):
    return int(os.stat(os.path.join(s.STATIC_ROOT, file)).st_mtime)

def format_attrs(attrs):
    """ Formats HTML key/value attributes::

        >>> format_attrs([("foo", "bar"), ("a", '"bar"')])
        ' foo="bar" a="&qt;bar&qt;"'
        """
    return (attrs and " " or "") + " ".join(
        '%s="%s"' %(key, escape(val)) for (key, val) in attrs
    )

def _mtime_suffix(file):
    try:
        return _stat(file)
    except OSError as e:
        if e.errno != 2: # no such file or directory
            raise

    staticfiles = find_staticfiles(file, all=True)
    if not staticfiles:
        raise Exception("no static files found matching %r" %(file, ))
    if len(staticfiles) > 2:
        raise Exception("multiple static files found matching %r: %r"
                        %(file, staticfiles))
    return _stat(staticfiles[0])

@register.simple_tag(takes_context=True)
def static_url(context, file, absolute=False):
    url = "%s%s?%s" %(s.STATIC_URL, file, _mtime_suffix(file))
    if absolute:
        url = context["request"].build_absolute_uri(url)
    return url

@register.simple_tag
def stylesheet_link(stylesheet, media="all", rel="stylesheet", type="text/css",
                    **attrs):
    return mark_safe('<link rel="%s" href="%s%s?%d" type="%s" media="%s"%s />' %(
        rel,
        s.STATIC_URL,
        stylesheet,
        _mtime_suffix(stylesheet),
        type,
        media,
        format_attrs(attrs.items()),
    ))

@register.simple_tag
def script_link(script, type="text/javascript", **attrs):
    return mark_safe('<script type="%s" src="%s%s?%d"%s></script>' %(
        type,
        s.STATIC_URL,
        script,
        _mtime_suffix(script),
        format_attrs(attrs.items()),
    ))



import re
import os
import mimetypes
from cStringIO import StringIO

from django.utils.http import http_date
from django.views.static import was_modified_since
from django.http import HttpResponseNotModified
try:
    from django.http import CompatibleStreamingHttpResponse as StreamingHttpResponse
except ImportError:
    from django.http import StreamingHttpResponse

def escape_header(header_value):
    """ Escapes an HTTP header (ex, a ``Content-Disposition``).
        Kind of blunt and probably not 100% correct... But should be safe. """
    return re.sub(" *[\t\n\r;,]+ *", " ", header_value)


def data_response(data, size=None,
                  attachment=False, attachment_name=None,
                  mimetype=None, encoding=None):
    if hasattr(data, "getvalue"):
        data = data.getvalue()
    if isinstance(data, unicode):
        data = data.encode("utf-8")
        encoding = "utf-8"
    if isinstance(data, str):
        size = len(data)
        data = StringIO(data)
    mimetype = mimetype or 'application/octet-stream'
    response = StreamingHttpResponse(data, content_type=mimetype)
    if size is not None:
        response["Content-Length"] = size
    if encoding:
        response["Content-Encoding"] = encoding
    if attachment or attachment_name:
        response["Content-Disposition"] = "attachment" + (
            attachment_name and
            "; filename=%s" %(escape_header(attachment_name), ) or ""
        )
    return response


def file_response(request, filepath,
                  attachment=False, attachment_name=None,
                  mimetype=None, encoding=None):
    statobj = os.stat(filepath)
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                              statobj.st_mtime, statobj.st_size):
        return HttpResponseNotModified()

    if mimetype is None:
        mimetype, encoding = mimetypes.guess_type(filepath)
    response = data_response(
        open(filepath, 'rb'), size=statobj.st_size,
        mimetype=mimetype, encoding=encoding,
    )
    response["Last-Modified"] = http_date(statobj.st_mtime)
    return response



from django.template import Library

register = Library()

def to_int(s, default=0):
    """
    Return input converted into an integer. If failed, then return ``default``.

    Examples::

        >>> to_int('1')
        1
        >>> to_int(1)
        1
        >>> to_int('')
        0
        >>> to_int(None)
        0
        >>> to_int(0, default='Empty')
        0
        >>> to_int(None, default='Empty')
        'Empty'
    """
    try:
        return int(s)
    except (TypeError, ValueError):
        return default

@register.simple_tag
def plural(n_str, singular, plural=None):
    """ A better pluralization template tag.

        The syntax is ``{% plural number "singular" "plural" %}``, where the
        ``plural`` is optional (the ``singular`` with an ``"s"`` suffix
        will be used if none is provided).

        By default numbers will be formatted using the ``{:,}`` formatter, so
        they will include a comma: ``1,234,567``.

        If the ``singular`` and ``plural`` strings can contain a ``{``, they
        will be treated as ``str.format`` templates::
        
            > There {% plural cats|length "is {} cat" "are {} cats" %}.
            There is 1 cat.
            > There {% plural dogs|length "is {} dog" "are {} dogs" %}.
            There are 4 dogs.

        Unlike Django's ``pluralize`` filter, ``plural`` does *not* take the
        length of lists; the ``|length`` filter can be used instead::

            > You have {% friends "friend" %}.
            You have ['Alex'] friends.
            > You have {% friends|length "friend" %}.
            You have 1 friend.

        Examples::

            > I have {% plural dog_count "dog" %}.
            I have 3 dogs.

            > You have {% plural ox_count "ox" "oxen" %}
            You have 1 ox.

            > There {% plural cats|length "is {} cat" "are {} cats" %}!
            There are 16 cats!

            > The plural will save you {% plural hours "hour" %}.
            The plural tag will save you 12,345 hours.
        """

    n = to_int(n_str, default=None)

    if plural is None:
        plural = singular + u"s"

    formatstr = singular if n == 1 else plural
    if "{" not in formatstr:
        default_format = u"{:,} " if n is not None else u"{} "
        formatstr = default_format + formatstr

    return formatstr.format(n_str)
